/** @file src/table/widget.c Widget file table. */

#include <stdio.h>
#include "types.h"

#include "../gui/widget.h"
#include "strings.h"

Widget g_table_windowWidgets[9] = {
	{ /* 0 */
		/* next                  */ NULL,
		/* index                 */ 30,
		/* shortcut              */ 0,
		/* shortcut2             */ 0,
		/* drawModeNormal        */ 1,
		/* drawModeSelected      */ 1,
		/* drawModeDown          */ 1,
		{ /* flags */
		/* requiresClick         */ true,
		/* notused1              */ true,
		/* clickAsHover          */ false,
		/* invisible             */ false,
		/* greyWhenInvisible     */ false,
		/* noClickCascade        */ false,
		/* loseSelect            */ true,
		/* notused2              */ false,
		/* buttonFilterLeft      */ 4,
		/* buttonFilterRight     */ 4
		},
		/* drawProcNormal        */ { 0x0 },
		/* drawProcSelected      */ { 0x0 },
		/* drawProcDown          */ { 0x0 },
		/* parentID              */ 0,
		/* offsetX               */ 0,
		/* offsetY               */ 0,
		/* width                 */ 0,
		/* height                */ 0,
		/* fgColourNormal        */ 15,
		/* bgColourNormal        */ 12,
		/* fgColourSelected      */ 15,
		/* bgColourSelected      */ 12,
		/* fgColourDown          */ 15,
		/* bgColourDown          */ 12,
		{ /* state */
		/* selected              */ false,
		/* hover1                */ false,
		/* hover2                */ false,
		/* selectedLast          */ false,
		/* hover1Last            */ false,
		/* hover2Last            */ false,
		/* notused               */ false,
		/* keySelected           */ false,
		/* buttonState           */ 0,
		},
		/* clickProc             */ NULL,
		/* data                  */ NULL,
		/* stringID              */ STR_NULL,
	},
	{ /* 1 */
		/* next                  */ NULL,
		/* index                 */ 31,
		/* shortcut              */ 0,
		/* shortcut2             */ 0,
		/* drawModeNormal        */ 1,
		/* drawModeSelected      */ 1,
		/* drawModeDown          */ 1,
		{ /* flags */
		/* requiresClick         */ true,
		/* notused1              */ true,
		/* clickAsHover          */ false,
		/* invisible             */ false,
		/* greyWhenInvisible     */ false,
		/* noClickCascade        */ false,
		/* loseSelect            */ true,
		/* notused2              */ false,
		/* buttonFilterLeft      */ 4,
		/* buttonFilterRight     */ 4
		},
		/* drawProcNormal        */ { 0x0 },
		/* drawProcSelected      */ { 0x0 },
		/* drawProcDown          */ { 0x0 },
		/* parentID              */ 0,
		/* offsetX               */ 0,
		/* offsetY               */ 0,
		/* width                 */ 0,
		/* height                */ 0,
		/* fgColourNormal        */ 15,
		/* bgColourNormal        */ 12,
		/* fgColourSelected      */ 15,
		/* bgColourSelected      */ 12,
		/* fgColourDown          */ 15,
		/* bgColourDown          */ 12,
		{ /* state */
		/* selected              */ false,
		/* hover1                */ false,
		/* hover2                */ false,
		/* selectedLast          */ false,
		/* hover1Last            */ false,
		/* hover2Last            */ false,
		/* notused               */ false,
		/* keySelected           */ false,
		/* buttonState           */ 0,
		},
		/* clickProc             */ NULL,
		/* data                  */ NULL,
		/* stringID              */ STR_NULL,
	},
	{ /* 2 */
		/* next                  */ NULL,
		/* index                 */ 32,
		/* shortcut              */ 0,
		/* shortcut2             */ 0,
		/* drawModeNormal        */ 1,
		/* drawModeSelected      */ 1,
		/* drawModeDown          */ 1,
		{ /* flags */
		/* requiresClick         */ true,
		/* notused1              */ true,
		/* clickAsHover          */ false,
		/* invisible             */ false,
		/* greyWhenInvisible     */ false,
		/* noClickCascade        */ false,
		/* loseSelect            */ true,
		/* notused2              */ false,
		/* buttonFilterLeft      */ 4,
		/* buttonFilterRight     */ 4
		},
		/* drawProcNormal        */ { 0x0 },
		/* drawProcSelected      */ { 0x0 },
		/* drawProcDown          */ { 0x0 },
		/* parentID              */ 0,
		/* offsetX               */ 0,
		/* offsetY               */ 0,
		/* width                 */ 0,
		/* height                */ 0,
		/* fgColourNormal        */ 15,
		/* bgColourNormal        */ 12,
		/* fgColourSelected      */ 15,
		/* bgColourSelected      */ 12,
		/* fgColourDown          */ 15,
		/* bgColourDown          */ 12,
		{ /* state */
		/* selected              */ false,
		/* hover1                */ false,
		/* hover2                */ false,
		/* selectedLast          */ false,
		/* hover1Last            */ false,
		/* hover2Last            */ false,
		/* notused               */ false,
		/* keySelected           */ false,
		/* buttonState           */ 0,
		},
		/* clickProc             */ NULL,
		/* data                  */ NULL,
		/* stringID              */ STR_NULL,
	},
	{ /* 3 */
		/* next                  */ NULL,
		/* index                 */ 33,
		/* shortcut              */ 0,
		/* shortcut2             */ 0,
		/* drawModeNormal        */ 1,
		/* drawModeSelected      */ 1,
		/* drawModeDown          */ 1,
		{ /* flags */
		/* requiresClick         */ true,
		/* notused1              */ true,
		/* clickAsHover          */ false,
		/* invisible             */ false,
		/* greyWhenInvisible     */ false,
		/* noClickCascade        */ false,
		/* loseSelect            */ true,
		/* notused2              */ false,
		/* buttonFilterLeft      */ 4,
		/* buttonFilterRight     */ 4
		},
		/* drawProcNormal        */ { 0x0 },
		/* drawProcSelected      */ { 0x0 },
		/* drawProcDown          */ { 0x0 },
		/* parentID              */ 0,
		/* offsetX               */ 0,
		/* offsetY               */ 0,
		/* width                 */ 0,
		/* height                */ 0,
		/* fgColourNormal        */ 15,
		/* bgColourNormal        */ 12,
		/* fgColourSelected      */ 15,
		/* bgColourSelected      */ 12,
		/* fgColourDown          */ 15,
		/* bgColourDown          */ 12,
		{ /* state */
		/* selected              */ false,
		/* hover1                */ false,
		/* hover2                */ false,
		/* selectedLast          */ false,
		/* hover1Last            */ false,
		/* hover2Last            */ false,
		/* notused               */ false,
		/* keySelected           */ false,
		/* buttonState           */ 0,
		},
		/* clickProc             */ NULL,
		/* data                  */ NULL,
		/* stringID              */ STR_NULL,
	},
	{ /* 4 */
		/* next                  */ NULL,
		/* index                 */ 34,
		/* shortcut              */ 0,
		/* shortcut2             */ 0,
		/* drawModeNormal        */ 1,
		/* drawModeSelected      */ 1,
		/* drawModeDown          */ 1,
		{ /* flags */
		/* requiresClick         */ true,
		/* notused1              */ true,
		/* clickAsHover          */ false,
		/* invisible             */ false,
		/* greyWhenInvisible     */ false,
		/* noClickCascade        */ false,
		/* loseSelect            */ true,
		/* notused2              */ false,
		/* buttonFilterLeft      */ 4,
		/* buttonFilterRight     */ 4
		},
		/* drawProcNormal        */ { 0x0 },
		/* drawProcSelected      */ { 0x0 },
		/* drawProcDown          */ { 0x0 },
		/* parentID              */ 0,
		/* offsetX               */ 0,
		/* offsetY               */ 0,
		/* width                 */ 0,
		/* height                */ 0,
		/* fgColourNormal        */ 15,
		/* bgColourNormal        */ 12,
		/* fgColourSelected      */ 15,
		/* bgColourSelected      */ 12,
		/* fgColourDown          */ 15,
		/* bgColourDown          */ 12,
		{ /* state */
		/* selected              */ false,
		/* hover1                */ false,
		/* hover2                */ false,
		/* selectedLast          */ false,
		/* hover1Last            */ false,
		/* hover2Last            */ false,
		/* notused               */ false,
		/* keySelected           */ false,
		/* buttonState           */ 0,
		},
		/* clickProc             */ NULL,
		/* data                  */ NULL,
		/* stringID              */ STR_NULL,
	},
	{ /* 5 */
		/* next                  */ NULL,
		/* index                 */ 35,
		/* shortcut              */ 0,
		/* shortcut2             */ 0,
		/* drawModeNormal        */ 1,
		/* drawModeSelected      */ 1,
		/* drawModeDown          */ 1,
		{ /* flags */
		/* requiresClick         */ true,
		/* notused1              */ true,
		/* clickAsHover          */ false,
		/* invisible             */ false,
		/* greyWhenInvisible     */ false,
		/* noClickCascade        */ false,
		/* loseSelect            */ true,
		/* notused2              */ false,
		/* buttonFilterLeft      */ 4,
		/* buttonFilterRight     */ 4
		},
		/* drawProcNormal        */ { 0x0 },
		/* drawProcSelected      */ { 0x0 },
		/* drawProcDown          */ { 0x0 },
		/* parentID              */ 0,
		/* offsetX               */ 0,
		/* offsetY               */ 0,
		/* width                 */ 0,
		/* height                */ 0,
		/* fgColourNormal        */ 15,
		/* bgColourNormal        */ 12,
		/* fgColourSelected      */ 15,
		/* bgColourSelected      */ 12,
		/* fgColourDown          */ 15,
		/* bgColourDown          */ 12,
		{ /* state */
		/* selected              */ false,
		/* hover1                */ false,
		/* hover2                */ false,
		/* selectedLast          */ false,
		/* hover1Last            */ false,
		/* hover2Last            */ false,
		/* notused               */ false,
		/* keySelected           */ false,
		/* buttonState           */ 0,
		},
		/* clickProc             */ NULL,
		/* data                  */ NULL,
		/* stringID              */ STR_NULL,
	},
	{ /* 6 */
		/* next                  */ NULL,
		/* index                 */ 36,
		/* shortcut              */ 0,
		/* shortcut2             */ 0,
		/* drawModeNormal        */ 1,
		/* drawModeSelected      */ 1,
		/* drawModeDown          */ 1,
		{ /* flags */
		/* requiresClick         */ true,
		/* notused1              */ true,
		/* clickAsHover          */ false,
		/* invisible             */ false,
		/* greyWhenInvisible     */ false,
		/* noClickCascade        */ false,
		/* loseSelect            */ true,
		/* notused2              */ false,
		/* buttonFilterLeft      */ 4,
		/* buttonFilterRight     */ 4
		},
		/* drawProcNormal        */ { 0x0 },
		/* drawProcSelected      */ { 0x0 },
		/* drawProcDown          */ { 0x0 },
		/* parentID              */ 0,
		/* offsetX               */ 0,
		/* offsetY               */ 0,
		/* width                 */ 0,
		/* height                */ 0,
		/* fgColourNormal        */ 15,
		/* bgColourNormal        */ 12,
		/* fgColourSelected      */ 15,
		/* bgColourSelected      */ 12,
		/* fgColourDown          */ 15,
		/* bgColourDown          */ 12,
		{ /* state */
		/* selected              */ false,
		/* hover1                */ false,
		/* hover2                */ false,
		/* selectedLast          */ false,
		/* hover1Last            */ false,
		/* hover2Last            */ false,
		/* notused               */ false,
		/* keySelected           */ false,
		/* buttonState           */ 0,
		},
		/* clickProc             */ NULL,
		/* data                  */ NULL,
		/* stringID              */ STR_NULL,
	},
	{ /* 7 */
		/* next                  */ NULL,
		/* index                 */ 37,
		/* shortcut              */ 96,
		/* shortcut2             */ 0,
		/* drawModeNormal        */ 1,
		/* drawModeSelected      */ 1,
		/* drawModeDown          */ 1,
		{ /* flags */
		/* requiresClick         */ true,
		/* notused1              */ true,
		/* clickAsHover          */ true,
		/* invisible             */ false,
		/* greyWhenInvisible     */ false,
		/* noClickCascade        */ false,
		/* loseSelect            */ false,
		/* notused2              */ true,
		/* buttonFilterLeft      */ 4,
		/* buttonFilterRight     */ 4
		},
		/* drawProcNormal        */ { 0x0 },
		/* drawProcSelected      */ { 0x0 },
		/* drawProcDown          */ { 0x0 },
		/* parentID              */ 17,
		/* offsetX               */ 128,
		/* offsetY               */ 20,
		/* width                 */ 25,
		/* height                */ 16,
		/* fgColourNormal        */ 15,
		/* bgColourNormal        */ 12,
		/* fgColourSelected      */ 15,
		/* bgColourSelected      */ 12,
		/* fgColourDown          */ 15,
		/* bgColourDown          */ 12,
		{ /* state */
		/* selected              */ false,
		/* hover1                */ false,
		/* hover2                */ false,
		/* selectedLast          */ false,
		/* hover1Last            */ false,
		/* hover2Last            */ false,
		/* notused               */ false,
		/* keySelected           */ false,
		/* buttonState           */ 0,
		},
		/* clickProc             */ NULL,
		/* data                  */ NULL,
		/* stringID              */ STR_NULL,
	},
	{ /* 8 */
		/* next                  */ NULL,
		/* index                 */ 38,
		/* shortcut              */ 98,
		/* shortcut2             */ 0,
		/* drawModeNormal        */ 1,
		/* drawModeSelected      */ 1,
		/* drawModeDown          */ 1,
		{ /* flags */
		/* requiresClick         */ true,
		/* notused1              */ true,
		/* clickAsHover          */ true,
		/* invisible             */ false,
		/* greyWhenInvisible     */ false,
		/* noClickCascade        */ false,
		/* loseSelect            */ false,
		/* notused               */ true,
		/* buttonFilterLeft      */ 4,
		/* buttonFilterRight     */ 4
		},
		/* drawProcNormal        */ { 0x0 },
		/* drawProcSelected      */ { 0x0 },
		/* drawProcDown          */ { 0x0 },
		/* parentID              */ 17,
		/* offsetX               */ 128,
		/* offsetY               */ 126,
		/* width                 */ 25,
		/* height                */ 16,
		/* fgColourNormal        */ 15,
		/* bgColourNormal        */ 12,
		/* fgColourSelected      */ 15,
		/* bgColourSelected      */ 12,
		/* fgColourDown          */ 15,
		/* bgColourDown          */ 12,
		{ /* state */
		/* selected              */ false,
		/* hover1                */ false,
		/* hover2                */ false,
		/* selectedLast          */ false,
		/* hover1Last            */ false,
		/* hover2Last            */ false,
		/* notused               */ false,
		/* keySelected           */ false,
		/* buttonState           */ 0,
		},
		/* clickProc             */ NULL,
		/* data                  */ NULL,
		/* stringID              */ STR_NULL,
	}
};
