/** @file src/codec/format80.h Function signature for 'format80' files. */

#ifndef CODEC_FORMAT80_H
#define CODEC_FORMAT80_H

uint16 Format80_Decode(uint8 *dest, const uint8 *source, uint16 destLength);

#endif /* CODEC_FORMAT80_H */
